/* packet-rtpmidi.c
 *
 * Routines for RFC xxxx RTP-MIDI dissection
 * Copyright 2003, Tobias Erichsen <t.erichsen@gmx.de>
 *
 * $Id: packet-rtpmidi.c 18196 2006-05-21 04:49:01Z sahlberg $
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * This dissector tries to dissect RTP-MIDI.
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/prefs.h>

#include <stdio.h>
#include <string.h>
#include <epan/conversation.h>
#include <epan/tap.h>

/*  rtp_midi_payload_type_value is the value used globally
	to set the appropriate payload type
    saved_pt_value is a temporary place to save the value
    	so we can properly reinitialize when the settings
    	get changed
*/
static guint rtp_midi_payload_type_value = 97;
static guint saved_payload_type_value;


/* RTP Event Fields */

static int proto_rtp_midi          = -1;
static int hf_rtp_midi_bflag       = -1;
static int hf_rtp_midi_jflag       = -1;
static int hf_rtp_midi_zflag       = -1;
static int hf_rtp_midi_pflag       = -1;
static int hf_rtp_midi_shortlen    = -1;
static int hf_rtp_midi_longlen     = -1;

static int hf_rtp_midi_sflag          = -1;
static int hf_rtp_midi_yflag          = -1;
static int hf_rtp_midi_aflag          = -1;
static int hf_rtp_midi_hflag          = -1;
static int hf_rtp_midi_totchan        = -1;
static int hf_rtp_midi_check_seq_num  = -1;

static int hf_rtp_midi_deltatime1     = -1;
static int hf_rtp_midi_deltatime2     = -1;
static int hf_rtp_midi_deltatime3     = -1;
static int hf_rtp_midi_deltatime4     = -1;

static int hf_rtp_midi_status         = -1;
static int hf_rtp_midi_rsstatus		  = -1;
static int hf_rtp_midi_command		  = -1;
static int hf_rtp_midi_rscommand	  = -1;
static int hf_rtp_midi_channel        = -1;
static int hf_rtp_midi_rschannel      = -1;
static int hf_rtp_midi_note			  = -1;
static int hf_rtp_midi_velocity       = -1;
static int hf_rtp_midi_pressure       = -1;
//static int hf_rtp_midi_controller	  = -1;
static int hf_rtp_midi_controller_value =-1;
static int hf_rtp_midi_program			=-1;
static int hf_rtp_midi_channel_pressure =-1;

static int hf_rtp_midi_sysjour_toc_s  =-1;
static int hf_rtp_midi_sysjour_toc_d  =-1;
static int hf_rtp_midi_sysjour_toc_v  =-1;
static int hf_rtp_midi_sysjour_toc_q  =-1;
static int hf_rtp_midi_sysjour_toc_f  =-1;
static int hf_rtp_midi_sysjour_toc_x  =-1;
static int hf_rtp_midi_sysjour_len  =-1;

static int hf_rtp_midi_chanjour_sflag =-1;
static int hf_rtp_midi_chanjour_chan=-1;
static int hf_rtp_midi_chanjour_hflag =-1;
static int hf_rtp_midi_chanjour_len =-1;
static int hf_rtp_midi_chanjour_toc_p =-1;
static int hf_rtp_midi_chanjour_toc_c =-1;
static int hf_rtp_midi_chanjour_toc_m =-1;
static int hf_rtp_midi_chanjour_toc_w=-1;
static int hf_rtp_midi_chanjour_toc_n=-1;
static int hf_rtp_midi_chanjour_toc_e=-1;
static int hf_rtp_midi_chanjour_toc_t=-1;
static int hf_rtp_midi_chanjour_toc_a=-1;

static int hf_rtp_midi_sysex_chan = -1;






/* RTP MIDI fields defining a subtree */

static gint ett_rtp_midi           = -1;
static gint ett_rtp_midi_commands  = -1;
static gint ett_rtp_midi_journal   = -1;
static gint ett_rtp_midi_command   = -1;
static gint ett_rtp_midi_systemjournal = -1;
static gint ett_rtp_midi_channeljournals = -1;
static gint ett_rtp_midi_systemchapters = -1;
static gint ett_rtp_midi_channeljournal =-1;
static gint ett_rtp_midi_channelchapters = -1;
static gint ett_rtp_midi_sysex_data = -1;

void
proto_reg_handoff_rtp_midi(void);


char *get_controller(char *buffer, guint8 ctrl, guint8 value)
{
	switch(ctrl) {
		case 0:
			strcpy(buffer,"Bank Select (coarse)");
			break;
		case 1:
			strcpy(buffer,"Modulation Wheel (coarse)");
			break;
		case 2:
			strcpy(buffer,"Breath Controller (coarse)");
			break;
		case 4:
			strcpy(buffer,"Foot Pedal (coarse)");
			break;
		case 5:
			strcpy(buffer,"Portamento Time (coarse)");
			break;
		case 6:
			strcpy(buffer,"Data Entry (coarse)");
			break;
		case 7:
			strcpy(buffer,"Volume (coarse)");
			break;
		case 8:
			strcpy(buffer,"Balance (coarse)");
			break;
		case 10:
			strcpy(buffer,"Pan Position (coarse)");
			break;
		case 11:
			strcpy(buffer,"Expression (coarse)");
			break;
		case 12:
			strcpy(buffer,"Effect Control 1 (coarse)");
			break;
		case 13:
			strcpy(buffer,"Effect Control 2 (coarse)");
			break;
		case 16:
			strcpy(buffer,"General Purpose Slider 1");
			break;
		case 17:
			strcpy(buffer,"General Purpose Slider 2");
			break;
		case 18:
			strcpy(buffer,"General Purpose Slider 3");
			break;
		case 19:
			strcpy(buffer,"General Purpose Slider 4");
			break;
		case 32:
			strcpy(buffer,"Bank Select (fine)");
			break;
		case 33:
			strcpy(buffer,"Modulation Wheel (fine)");
			break;
		case 34:
			strcpy(buffer,"Breath Controller (fine)");
			break;
		case 36:
			strcpy(buffer,"Foot Pedal (fine)");
			break;
		case 37:
			strcpy(buffer,"Portamento Time (fine)");
			break;
		case 38:
			strcpy(buffer,"Data Entry (fine)");
			break;
		case 39:
			strcpy(buffer,"Volume (fine)");
			break;
		case 40:
			strcpy(buffer,"Balance (fine)");
			break;
		case 42:
			strcpy(buffer,"Pan Position (fine)");
			break;
		case 43:
			strcpy(buffer,"Expression (fine)");
			break;
		case 44:
			strcpy(buffer,"Effect Control 1 (fine)");
			break;
		case 45:
			strcpy(buffer,"Effect Control 2 (fine)");
			break;
		case 64:
			if (value<=63)
			  strcpy(buffer,"Hold Pedal (off)");
			else
			  strcpy(buffer,"Hold Pedal (on)");
			break;
		case 65:
			if (value<=63)
			  strcpy(buffer,"Portamento (off)");
			else
			  strcpy(buffer,"Portamento (on)");
			break;
		case 66:
			if (value<=63)
			  strcpy(buffer,"Sustenuto Pedal (off)");
			else
			  strcpy(buffer,"Sustenuto Pedal (on)");
			break;
		case 67:
			if (value<=63)
			  strcpy(buffer,"Soft Pedal (off)");
			else
			  strcpy(buffer,"Soft Pedal (on)");
			break;
		case 68:
			if (value<=63)
			  strcpy(buffer,"Legato Pedal (off)");
			else
			  strcpy(buffer,"Legato Pedal (on)");
			break;
		case 69:
			if (value<=63)
			  strcpy(buffer,"Hold 2 Pedal (off)");
			else
			  strcpy(buffer,"Hold 2 Pedal (on)");
			break;
		case 70:
			strcpy(buffer,"Sound Variation");
			break;
		case 71:
			strcpy(buffer,"Sound Timbre");
			break;
		case 72:
			strcpy(buffer,"Sound Release Time");
			break;
		case 73:
			strcpy(buffer,"Sound Attack Time");
			break;
		case 74:
			strcpy(buffer,"Sound Brightness");
			break;
		case 75:
			strcpy(buffer,"Sound Control 6");
			break;
		case 76:
			strcpy(buffer,"Sound Control 7");
			break;
		case 77:
			strcpy(buffer,"Sound Control 8");
			break;
		case 78:
			strcpy(buffer,"Sound Control 9");
			break;
		case 79:
			strcpy(buffer,"Sound Control 10");
			break;
		case 80:
			if (value<=63)
			  strcpy(buffer,"General Purpose Button 1 (off)");
			else
			  strcpy(buffer,"General Purpose Button 1 (on)");
			break; 
		case 81:
			if (value<=63)
			  strcpy(buffer,"General Purpose Button 2 (off)");
			else
			  strcpy(buffer,"General Purpose Button 2 (on)");
			break;
		case 82:
			if (value<=63)
			  strcpy(buffer,"General Purpose Button 3 (off)");
			else
			  strcpy(buffer,"General Purpose Button 3 (on)");
			break;
		case 83:
			if (value<=63)
			  strcpy(buffer,"General Purpose Button 4 (off)");
			else
			  strcpy(buffer,"General Purpose Button 4 (on)");
			break;
		case 91:
			strcpy(buffer,"Effects Level");
			break;
		case 92:
			strcpy(buffer,"Tremolo Level");
			break;
		case 93:
			strcpy(buffer,"Chorus Level");
			break;
		case 94:
			strcpy(buffer,"Celeste Level");
			break;
		case 95:
			strcpy(buffer,"Phaser Level");
			break;
		case 96:
			strcpy(buffer,"Data Button Increment");
			break;
		case 97:
			strcpy(buffer,"Data Button Decrement");
			break;
		case 98:
			strcpy(buffer,"Non-registered Parameter (fine)");
			break;
		case 99:
			strcpy(buffer,"Non-registered Parameter (coarse)");
			break;
		case 100:
			strcpy(buffer,"Registered Parameter (fine)");
			break;
		case 101:
			strcpy(buffer,"Registered Parameter (coarse)");
			break;
		case 120:
			strcpy(buffer,"All Sound Off");
			break;
		case 121:
			strcpy(buffer,"All Controllers Off");
			break;
		case 122:
			if (value<=63)
			  strcpy(buffer,"Local Keyboard (off)");
			else
			  strcpy(buffer,"Local Keyboard (on)");
			break;
		case 123:
			strcpy(buffer,"All Notes Off");
			break;
		case 124:
			strcpy(buffer,"Omni Mode Off");
			break;
		case 125:
			strcpy(buffer,"Omni Mode On");
			break;
		case 126:
			strcpy(buffer,"Mono Operation");
			break;
		case 127:
			strcpy(buffer,"Poly Operation");
			break;
		default:
			sprintf(buffer,"Unknown: %d",ctrl);
	}
	return buffer;
}

char *get_short_manu(char *buffer, guint8 id)
{
	switch (id) {
		case 1:
			strcpy(buffer,"Sequential Circuits");
			break;
		case 2:
			strcpy(buffer,"Big Briar");
			break;
		case 3:
			strcpy(buffer,"Octave / Plateau");
			break;
		case 4:
			strcpy(buffer,"Moog");
			break;
		case 5:
			strcpy(buffer,"Passport Designs");
			break;
		case 6:
			strcpy(buffer,"Lexicon");
			break;
		case 7:
			strcpy(buffer,"Kurzweil");
			break;
		case 8:
			strcpy(buffer,"Fender");
			break;
		case 9:
			strcpy(buffer,"Gulbransen");
			break;
		case 10:
			strcpy(buffer,"Delta Labs");
			break;
		case 11:
			strcpy(buffer,"Sound Comp.");
			break;
		case 12:
			strcpy(buffer,"General Elector");
			break;
		case 13:
			strcpy(buffer,"Techmar");
			break;
		case 14:
			strcpy(buffer,"Matthews Research");
			break;
		case 16:
			strcpy(buffer,"Oberheim");
			break;
		case 17:
			strcpy(buffer,"Apple Computer");
			break;
		case 18:
			strcpy(buffer,"Simmons");
			break;
		case 19:
			strcpy(buffer,"DigiDesign");
			break;
		case 20:
			strcpy(buffer,"Fairlight");
			break;
		case 21:
			strcpy(buffer,"JL Cooper");
			break;
		case 22:
			strcpy(buffer,"Lowery");
			break;
		case 23:
			strcpy(buffer,"Lin");
			break;
		case 24:
			strcpy(buffer,"Emu");
			break;
		case 26:
			strcpy(buffer,"ART");
			break;
		case 32:
			strcpy(buffer,"Bon Tempi");
			break;
		case 33:
			strcpy(buffer,"S.I.E.L.");
			break;
		case 35:
			strcpy(buffer,"SyntheAxe");
			break;
		case 36:
			strcpy(buffer,"Hohner");
			break;
		case 37:
			strcpy(buffer,"Crumar");
			break;
		case 38:
			strcpy(buffer,"Solton");
			break;
		case 39:
			strcpy(buffer,"Jellinghaus Ms");
			break;
		case 40:
			strcpy(buffer,"CTS");
			break;
		case 41:
			strcpy(buffer,"PPG");
			break;
		case 43:
			strcpy(buffer,"SSL");
			break;
		case 47:
			strcpy(buffer,"Elka / General Music");
			break;
		case 48:
			strcpy(buffer,"Dynacord");
			break;
		case 54:
			strcpy(buffer,"Cheetah");
			break;
		case 62:
			strcpy(buffer,"Waldorf");
			break;
		case 64:
			strcpy(buffer,"Kawai");
			break;
		case 65:
			strcpy(buffer,"Roland");
			break;
		case 66:
			strcpy(buffer,"Korg");
			break;
		case 67:
			strcpy(buffer,"Yamaha");
			break;
		case 68:
			strcpy(buffer,"Casio");
			break;
		case 69:
			strcpy(buffer,"Akai");
			break;
		case 70:
			strcpy(buffer,"Victor");
			break;
		case 75:
			strcpy(buffer,"Fujitsu");
			break;
		case 76:
			strcpy(buffer,"Sony");
			break;
		case 78:
			strcpy(buffer,"TEAC");
			break;
		case 80:
			strcpy(buffer,"Matsushita Electric");
			break;
		case 81:
			strcpy(buffer,"Fostex");
			break;
		case 82:
			strcpy(buffer,"Zoom");
			break;
		case 84:
			strcpy(buffer,"Matsushita Communication");
			break;
		case 85:
			strcpy(buffer,"Suzuki");
			break;
		case 86:
			strcpy(buffer,"Fuji");
			break;
		case 87:
			strcpy(buffer,"Acoustic Technical Lab");
			break;
		case 89:
			strcpy(buffer,"Faith");
			break;
		case 90:
			strcpy(buffer,"Internet Corporation");
			break;
		case 92:
			strcpy(buffer,"Seekers Co");
			break;
		case 95:
			strcpy(buffer,"SD Card Assoc");
			break;
		case 125:
			strcpy(buffer,"Educational Use");
			break;
		case 126:
			strcpy(buffer,"Non-Realtime Universal");
			break;
		case 127:
			strcpy(buffer,"Realtime Universal");
			break;
		default:
			strcpy(buffer,"unknown");
	}
	return buffer;
}

char *get_long_manu(char *buffer, guint16 id)
{
	switch (id) {
		case 7:
			strcpy(buffer,"Digital Music Corporation");
			break;
		case 14:
			strcpy(buffer,"Alesis");
			break;
		case 21:
			strcpy(buffer,"KAT");
			break;
		case 22:
			strcpy(buffer,"Opcode");
			break;
		case 26:
			strcpy(buffer,"Allen & Heath Brenell");
			break;
		case 27:
			strcpy(buffer,"Peavey");
			break;
		case 28:
			strcpy(buffer,"360 Systems");
			break;
		case 32:
			strcpy(buffer,"Axxes");
			break;
		case 65:
			strcpy(buffer,"Microsoft");
			break;
		case 0x74:
			strcpy(buffer,"Ta Hrong Musical Instrument");
			break;
		case 0x75:
			strcpy(buffer,"e-Tek Labs (Forte Tech)");
			break;
		case 0x76:
			strcpy(buffer,"Electro-Voice");
			break;
		case 0x77:
			strcpy(buffer,"Midisoft Corporation");
			break;
		case 0x78:
			strcpy(buffer,"QSound Labs");
			break;
		case 0x79:
			strcpy(buffer,"Westrex");
			break;
		case 0x7a:
			strcpy(buffer,"NVidia");
			break;
		case 0x7b:
			strcpy(buffer,"ESS Technology");
			break;
		case 0x7c:
			strcpy(buffer,"MediaTrix Peripherals");
			break;
		case 0x7d:
			strcpy(buffer,"Brooktree Corp");
			break;
		case 0x7e:
			strcpy(buffer,"Otari Corp");
			break;
		case 0x7f:
			strcpy(buffer,"Key Electronics");
			break;
		case 0x100:
			strcpy(buffer,"Shure Incorporated");
			break;
		case 0x101:
			strcpy(buffer,"Aura Sound");
			break;
		case 0x102:
			strcpy(buffer,"Crystal Semiconductor");
			break;
		case 0x103:
			strcpy(buffer,"Conexant (Rockwell)");
			break;
		case 0x104:
			strcpy(buffer,"Silicon Graphics");
			break;
		case 0x105:
			strcpy(buffer,"M-Audio (Midiman");
			break;
		case 0x106:
			strcpy(buffer,"PreSonus");
			break;
		case 0x108:
			strcpy(buffer,"Topaz Enterprises");
			break;
		case 0x109:
			strcpy(buffer,"Cast Lighting");
			break;
		case 0x10a:
			strcpy(buffer,"Microsoft (Consumer Division)");
			break;
		case 0x10b:
			strcpy(buffer,"Sonic Foundry");
			break;
		case 0x10c:
			strcpy(buffer,"Line 6 (Fast Forward)");
			break;
		case 0x10d:
			strcpy(buffer,"Beatnik Inc");
			break;
		case 0x10e:
			strcpy(buffer,"Van Koevering Company");
			break;
		case 0x10f:
			strcpy(buffer,"Altech Systems");
			break;
		case 0x110:
			strcpy(buffer,"S&S Research");
			break;
		case 0x111:
			strcpy(buffer,"VLSI Technology");
			break;
		case 0x112:
			strcpy(buffer,"Chromatic Research");
			break;
		case 0x113:
			strcpy(buffer,"Sapphire");
			break;
		case 0x114:
			strcpy(buffer,"IDRC");
			break;
		case 0x115:
			strcpy(buffer,"Justonic Tuning");
			break;
		case 0x116:
			strcpy(buffer,"TorComp Research Inc");
			break;
		case 0x117:
			strcpy(buffer,"NewTek Inc");
			break;
		case 0x118:
			strcpy(buffer,"Sound Sculpture");
			break;
		case 0x119:
			strcpy(buffer,"Walker Technical");
			break;
		case 0x11a:
			strcpy(buffer,"Digital Harmony");
			break;
		case 0x11b:
			strcpy(buffer,"InVision Interactive");
			break;
		case 0x11c:
			strcpy(buffer,"T-Square Design");
			break;
		case 0x11d:
			strcpy(buffer,"Nemesys Music Technology");
			break;
		case 0x11e:
			strcpy(buffer,"DBX Professional (Harman Intl)");
			break;
		case 0x11f:
			strcpy(buffer,"Syndyne Corporation");
			break;
		case 0x120:
			strcpy(buffer,"Bitheadz");
			break;
		case 0x121:
			strcpy(buffer,"Cakewalk Music Software");
			break;
		case 0x122:
			strcpy(buffer,"Analog Devices");
			break;
		case 0x123:
			strcpy(buffer,"National Semiconductor");
			break;
		case 0x124:
			strcpy(buffer,"Boom Theory");
			break;
		case 0x125:
			strcpy(buffer,"Virtual DSP Corporation");
			break;
		case 0x126:
			strcpy(buffer,"Antares Systems");
			break;
		case 0x127:
			strcpy(buffer,"Angel Software");
			break;
		case 0x128:
			strcpy(buffer,"St Louis Music");
			break;
		case 0x129:
			strcpy(buffer,"Lyrrus dba G-VOX");
			break;
		case 0x12a:
			strcpy(buffer,"Ashley Audio Inc");
			break;
		case 0x12b:
			strcpy(buffer,"Vari-Lite Inc");
			break;
		case 0x12c:
			strcpy(buffer,"Summit Audio Inc");
			break;
		case 0x12d:
			strcpy(buffer,"Aureal Semiconductor Inc");
			break;
		case 0x12e:
			strcpy(buffer,"SeaSound LLC");
			break;
		case 0x12f:
			strcpy(buffer,"U.S. Robotics");
			break;
		case 0x130:
			strcpy(buffer,"Aurisis Research");
			break;
		case 0x131:
			strcpy(buffer,"Nearfield Multimedia");
			break;
		case 0x132:
			strcpy(buffer,"FM7 Inc");
			break;
		case 0x133:
			strcpy(buffer,"Swivel Systems");
			break;
		case 0x134:
			strcpy(buffer,"Hyperactive Audio Systems");
			break;
		case 0x135:
			strcpy(buffer,"MidiLite (Castle Studios Productions");
			break;
		case 0x136:
			strcpy(buffer,"Radikal Technologies");
			break;
		case 0x137:
			strcpy(buffer,"Roger Linn Design");
			break;
		case 0x138:
			strcpy(buffer,"TC-Helicon Vocal Technologies");
			break;
		case 0x139:
			strcpy(buffer,"Event Electronics");
			break;
		case 0x13a:
			strcpy(buffer,"Sonic Network Inc");
			break;
		case 0x13b:
			strcpy(buffer,"Realtime Music Solutions");
			break;
		case 0x13c:
			strcpy(buffer,"Apogee Digital");
			break;
		case 0x13d:
			strcpy(buffer,"Classical Organs, Inc");
			break;
		case 0x13e:
			strcpy(buffer,"Microtools Inc");
			break;
		case 0x13f:
			strcpy(buffer,"Numark Industries");
			break;
		case 0x140:
			strcpy(buffer,"Frontier Design Group LLC");
			break;
		case 0x141:
			strcpy(buffer,"Recordare LLC");
			break;
		case 0x142:
			strcpy(buffer,"Starr Labs");
			break;
		case 0x143:
			strcpy(buffer,"Voyager Sound Inc");
			break;
		case 0x144:
			strcpy(buffer,"Manifold Labs");
			break;
		case 0x145:
			strcpy(buffer,"Aviom Inc");
			break;
		case 0x146:
			strcpy(buffer,"Mixmeister Technology");
			break;
		case 0x147:
			strcpy(buffer,"Notation Software");
			break;
		case 0x148:
			strcpy(buffer,"Mercurial Communications");
			break;
		case 0x149:
			strcpy(buffer,"Wave Arts");
			break;
		case 0x14a:
			strcpy(buffer,"Logic Sequencing Devices");
			break;
		case 0x14b:
			strcpy(buffer,"Axess Electronics");
			break;
		case 0x14c:
			strcpy(buffer,"Muse Research");
			break;
		case 0x14d:
			strcpy(buffer,"Open Labs");
			break;
		case 0x14e:
			strcpy(buffer,"Guillemot R&D Inc");
			break;
		case 0x14f:
			strcpy(buffer,"Samson Technologies");
			break;
		case 0x150:
			strcpy(buffer,"Electronic Theatre Controls");
			break;
		case 0x151:
			strcpy(buffer,"Research In Motion");
			break;
		case 0x152:
			strcpy(buffer,"Mobileer");
			break;
		case 0x153:
			strcpy(buffer,"Synthogy");
			break;
		case 0x154:
			strcpy(buffer,"Lynx Studio Technology Inc");
			break;
		case 0x155:
			strcpy(buffer,"Damage Control Engineering LLC");
			break;
		case 0x156:
			strcpy(buffer,"Yost Engineering Inc");
			break;
		case 0x157:
			strcpy(buffer,"Brooks & Forsman Designs LLC");
			break;
		case 0x158:
			strcpy(buffer,"Magnekey");
			break;
		case 0x159:
			strcpy(buffer,"Garritan Corp");
			break;
		case 0x15a:
			strcpy(buffer,"Ploque Art et Technologie, Inc");
			break;
		case 0x15b:
			strcpy(buffer,"RJM Music Technology");
			break;
		case 0x15c:
			strcpy(buffer,"Custom Solutions Software");
			break;
		case 0x202b:
			strcpy(buffer,"Medeli Electronics Co");
			break;
		case 0x202c:
			strcpy(buffer,"Charlie Lab SRL");
			break;
		case 0x202d:
			strcpy(buffer,"Blue Chip Music Technology");
			break;
		case 0x202e:
			strcpy(buffer,"BEE OH Corp");
			break;
		case 0x202f:
			strcpy(buffer,"LG Semicon America");
			break;
		case 0x2030:
			strcpy(buffer,"TESI");
			break;
		case 0x2031:
			strcpy(buffer,"EMAGIC");
			break;
		case 0x2032:
			strcpy(buffer,"Behringer GmbH");
			break;
		case 0x2033:
			strcpy(buffer,"Access Music Electronics");
			break;
		case 0x2034:
			strcpy(buffer,"Synoptic");
			break;
		case 0x2035:
			strcpy(buffer,"Hanmesoft Corp");
			break;
		case 0x2036:
			strcpy(buffer,"Terratec Electronic GmbH");
			break;
		case 0x2037:
			strcpy(buffer,"Proel SpA");
			break;
		case 0x2038:
			strcpy(buffer,"IBK MIDI");
			break;
		case 0x2039:
			strcpy(buffer,"IRCAM");
			break;
		case 0x203a:
			strcpy(buffer,"Propellerhead Software");
			break;
		case 0x203b:
			strcpy(buffer,"Red Sound Systems Ltd");
			break;
		case 0x203c:
			strcpy(buffer,"Elektron ESI AB");
			break;
		case 0x203d:
			strcpy(buffer,"Sintefex Audio");
			break;
		case 0x203e:
			strcpy(buffer,"MAM (Music and More)");
			break;
		case 0x203f:
			strcpy(buffer,"Amsaro GmbH");
			break;
		case 0x2040:
			strcpy(buffer,"CDS Advanced Technology BV");
			break;
		case 0x2041:
			strcpy(buffer,"Touched By Sound GmbH");
			break;
		case 0x2042:
			strcpy(buffer,"DSP Arts");
			break;
		case 0x2043:
			strcpy(buffer,"Phil Rees Music Tech");
			break;
		case 0x2044:
			strcpy(buffer,"Stamer Musikanlagen GmbH");
			break;
		case 0x2045:
			strcpy(buffer,"Musical Muntaner S.A. dba Soundart");
			break;
		case 0x2046:
			strcpy(buffer,"C-Mexx Software");
			break;
		case 0x2047:
			strcpy(buffer,"Klavis Technologies");
			break;
		case 0x2048:
			strcpy(buffer,"Noteheads AB");
			break;
		case 0x2049:
			strcpy(buffer,"Algorithmix");
			break;
		case 0x204a:
			strcpy(buffer,"Skrydstrup R&D");
			break;
		case 0x204b:
			strcpy(buffer,"Professional Audio Company");
			break;
		case 0x204c:
			strcpy(buffer,"DBTECH/MadWaves");
			break;
		case 0x204d:
			strcpy(buffer,"Vermona");
			break;
		case 0x204e:
			strcpy(buffer,"Nokia");
			break;
		case 0x204f:
			strcpy(buffer,"Wave Idea");
			break;
		case 0x2050:
			strcpy(buffer,"Hartmann GmbH");
			break;
		case 0x2051:
			strcpy(buffer,"Lion's Track");
			break;
		case 0x2052:
			strcpy(buffer,"Analogue Systems");
			break;
		case 0x2053:
			strcpy(buffer,"Focal-JMlab");
			break;
		case 0x2054:
			strcpy(buffer,"Ringway Electronics");
			break;
		case 0x2055:
			strcpy(buffer,"Faith Technologies (Digiplug)");
			break;
		case 0x2056:
			strcpy(buffer,"Showworks");
			break;
		case 0x2057:
			strcpy(buffer,"Manikin Electronic");
			break;
		case 0x2058:
			strcpy(buffer,"1 Come Tech");
			break;
		case 0x2059:
			strcpy(buffer,"Phonic Corp");
			break;
		case 0x205a:
			strcpy(buffer,"Lake Technology");
			break;
		case 0x205b:
			strcpy(buffer,"Silansys Technologies");
			break;
		case 0x205c:
			strcpy(buffer,"Winbond Electronics");
			break;
		case 0x205d:
			strcpy(buffer,"Cinetix Medien und Interface GmbH");
			break;
		case 0x205e:
			strcpy(buffer,"A&G Soluzioni Digitali");
			break;
		case 0x205f:
			strcpy(buffer,"Sequentix Music Systems");
			break;
		case 0x2060:
			strcpy(buffer,"Oram Pro Audio");
			break;
		case 0x2061:
			strcpy(buffer,"Be4 Ltd");
			break;
		case 0x2062:
			strcpy(buffer,"Infection Music");
			break;
		case 0x2063:
			strcpy(buffer,"Central Music Co. (CME)");
			break;
		case 0x2064:
			strcpy(buffer,"GenoQs Machines");
			break;
		case 0x2065:
			strcpy(buffer,"Medialon");
			break;
		case 0x2066:
			strcpy(buffer,"Waves Audio Ltd");
			break;
		default:
			strcpy(buffer,"unknown");
	}
	return buffer;
}

static unsigned int
decodetime(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, unsigned int offset, unsigned int cmd_offset, unsigned int cmd_len)
{
	guint8			octet;
	unsigned int	consumed;
	unsigned int    i;
	unsigned long   deltatime;

	consumed=0;
	deltatime=0;


	for (i=0;i<4;i++) {

		if ((cmd_offset+consumed)>=cmd_len) {
			return 0;
		}

		octet = tvb_get_guint8(tvb, offset+cmd_offset+consumed );
		deltatime=(deltatime << 7) | (octet&0x7f);
		consumed++;

		if ((octet&0x80)!=0x80) {
			break;
		}
	}

	switch (consumed) {
		case 1: 
		    proto_tree_add_uint ( tree, hf_rtp_midi_deltatime1, tvb, offset+cmd_offset, consumed, deltatime);
			break;
		case 2: 
		    proto_tree_add_uint ( tree, hf_rtp_midi_deltatime2, tvb, offset+cmd_offset, consumed, deltatime);
			break;
		case 3: 
		    proto_tree_add_uint ( tree, hf_rtp_midi_deltatime3, tvb, offset+cmd_offset, consumed, deltatime);
			break;
		case 4: 
		    proto_tree_add_uint ( tree, hf_rtp_midi_deltatime4, tvb, offset+cmd_offset, consumed, deltatime);
			break;

	}

	return consumed;
}

static unsigned int
decodemidi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, unsigned int offset, unsigned int cmd_offset, unsigned int cmd_len, guint8 *runningstatus, unsigned int *rsoffset, char *text)
{
	guint8			octet;
	guint8			octet2;
	guint8			octet3;
	guint8			cmd;
	guint8			channel;
	unsigned int	consumed;
	proto_item		*command_item     = NULL;
	proto_tree		*command_tree     = NULL;
	proto_tree		*sysexdata_tree   = NULL;
	gboolean		foundend;
	gboolean		usingrs;
    guint16         wordvalue;
	char			helptext[200];
	char			helptext2[200];
	char			entry[200];
	char			segtype[200];
	guint8			manu1;
	guint16			manu2;
	guint8			subid1;
	guint8			sysexchan;

	consumed=0;
	usingrs=FALSE;
	strcpy(text,"");

	if (cmd_offset>=cmd_len) {
		return 0;
	}

	octet = tvb_get_guint8(tvb, offset+cmd_offset);

	// see if we have a running status message
	if ((octet & 0x80)==0) {
		// if we have no running status yet -> error
		if (((*runningstatus) & 0x80)==0) {
			return 0;
		}
		octet=*runningstatus;
		usingrs=TRUE;
	} else {
		// we only have consumed anything yet, if we didnt have running status
		consumed++;
		// we need to store the location for the runningstatus to reference the correct octet in decoded-listing
		*rsoffset=offset+cmd_offset;
	}


	
	if (octet>=0xf8) {						// midi realtime-data -> one octet
		switch (octet) {
			case 0xf8:
				strcpy(text,"Realtime: Midi Clock");
				break;
			case 0xf9:
				strcpy(text,"Realtime: Midi Tick");
				break;
			case 0xfa:
				strcpy(text,"Realtime: Start song");
				break;
			case 0xfb:
				strcpy(text,"Realtime: Continue song");
				break;
			case 0xfc:
				strcpy(text,"Realtime: Stop song");
				break;
			case 0xfd:
				strcpy(text,"Realtime: Undefined");
				break;
			case 0xfe:
				strcpy(text,"Realtime: Active sensing");
				break;
			case 0xff:
				strcpy(text,"Realtime: System reset");
				break;
		}
		command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, 1, text);
		command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
	    proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);

	} else if (octet>=0xf0) {				// common-data
		*runningstatus=0;
		switch(octet) {
			case 0xf0:
				foundend=FALSE;
				while((cmd_offset+consumed)<cmd_len) {
					octet2 = tvb_get_guint8(tvb, offset+cmd_offset+consumed );
					if ((octet2 & 0x80)==0x80) {
						foundend=TRUE;
						switch (octet2) {
							case 0xf0:
							case 0xf7:
							case 0xf5:
								consumed++;
								if (octet2==0xf0) {
									strcpy(segtype,"Common: System-Exclusive (first segment)");
								} else if (octet2==0xf7){
									strcpy(segtype,"Common: System-Exclusive (complete segment)");
								} else {
									strcpy(segtype,"Common: System-Exclusive (complete segment, non 0xf7-status termination)");
								}
								strcpy(text,segtype);
								command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
								command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
								command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-2, "Sysex-data");
								if (consumed>2) {
									manu1 = tvb_get_guint8(tvb, offset+cmd_offset+1);
									switch (manu1) {
										case 0:
											if (consumed>4) {
												manu2=tvb_get_ntohs(tvb,offset+cmd_offset+2);
											    sysexdata_tree = proto_item_add_subtree( command_item, ett_rtp_midi_sysex_data);
											    sprintf(entry,"Manufacturer: %s",get_long_manu(helptext,manu2));
											    command_item = proto_tree_add_text(sysexdata_tree, tvb, offset+cmd_offset+1, 3, entry);
											    sprintf(text,"%s [manu=%s]",segtype,get_long_manu(helptext,manu2));
											}
											break;
										case 0x7e: // non realtime
											if (consumed>4) {
												sysexchan = tvb_get_guint8(tvb, offset+cmd_offset+2 );
												subid1=tvb_get_guint8(tvb,offset+cmd_offset+3);
												switch (subid1) {
													case 0x01: // Sample Dump Header
														sprintf(entry,"Universal NRT: Sample Dump Header");
														sprintf(text,"%s [Universal NRT: Sample Dump Header]",segtype);
														break;
													case 0x02: // Sample Dump Data Packet
														sprintf(entry,"Universal NRT: Sample Dump Data Packet");
														sprintf(text,"%s [Universal NRT: Sample Dump Data Packet]",segtype);
														break;
													case 0x03: // Sample Dump Request
														sprintf(entry,"Universal NRT: Sample Dump Request");
														sprintf(text,"%s [Universal NRT: Sample Dump Request]",segtype);
														break;
													case 0x04: // MTC Cueing
														sprintf(entry,"Universal NRT: MTC cueing");
														sprintf(text,"%s [Universal NRT: MTC cueing]",segtype);
														break;
													case 0x05: // Loop Points
														sprintf(entry,"Universal NRT: Loop Points");
														sprintf(text,"%s [Universal NRT: Loop Points]",segtype);
														break;
													case 0x06: // ID request
														sprintf(entry,"Universal NRT: ID Request");
														sprintf(text,"%s [Universal NRT: ID Request]",segtype);
														break;
													case 0x07: // File dump
														sprintf(entry,"Universal NRT: File dump");
														sprintf(text,"%s [Universal NRT: File dump]",segtype);
														break;
													case 0x08: // MIDI Tuning Standard
														sprintf(entry,"Universal: MIDI Tuning standard");
														sprintf(text,"%s [Universal NRT: MIDI Tuning standard]",segtype);
														break;
													case 0x09: // GM
														sprintf(entry,"Universal NRT: General Midi");
														sprintf(text,"%s [Universal NRT: General Midi]",segtype);
														break;
													case 0x0a: // DLS
														sprintf(entry,"Universal NRT: DLS");
														sprintf(text,"%s [Universal NRT: DLS]",segtype);
														break;
													case 0x7b: // End of file
														sprintf(entry,"Universal NRT: End of File");
														sprintf(text,"%s [Universal NRT: End of File]",segtype);
														break;
													case 0x7c: // Wait
														sprintf(entry,"Universal NRT: Wait");
														sprintf(text,"%s [Universal NRT: Wait]",segtype);
														break;
													case 0x7d: // Cancel
														sprintf(entry,"Universal NRT: Cancel");
														sprintf(text,"%s [Universal NRT: Cancel]",segtype);
														break;
													case 0x7e: // NAK
														sprintf(entry,"Universal NRT: NAK");
														sprintf(text,"%s [Universal NRT: NAK]",segtype);
														break;
													case 0x7f: // ACK
														sprintf(entry,"Universal NRT: ACK");
														sprintf(text,"%s [Universal NRT: ACK]",segtype);
														break;
													default:
														sprintf(entry,"Universal NRT: unknown (0x%02x)",subid1);
														sprintf(text,"%s [Universal NRT: unknown (0x%02x)]",segtype,subid1);
												}
												
												sysexdata_tree = proto_item_add_subtree( command_item, ett_rtp_midi_sysex_data);
												proto_tree_add_uint ( sysexdata_tree, hf_rtp_midi_sysex_chan, tvb, offset+cmd_offset+2, 1, sysexchan);
												command_item = proto_tree_add_text(sysexdata_tree, tvb, offset+cmd_offset+3, 1, entry);
											}
											break;
										case 0x7f: // realtime
											if (consumed>4) {
												sysexchan = tvb_get_guint8(tvb, offset+cmd_offset+2 );
												subid1=tvb_get_guint8(tvb,offset+cmd_offset+3);
												switch (subid1) {
													case 0x01: // Time Code
														sprintf(entry,"Universal RT: MIDI Time Code");
														sprintf(text,"%s [Universal RT: MIDI Time Code]",segtype);
														break;
													case 0x03: // Notation Information
														sprintf(entry,"Universal RT: Notation Information");
														sprintf(text,"%s [Universal RT: Notation Information]",segtype);
														break;
													case 0x04: // Device Control
														sprintf(entry,"Universal RT: Device Control");
														sprintf(text,"%s [Universal RT: Device Control]",segtype);
														break;
													case 0x05: // MTC Cueing
														sprintf(entry,"Universal RT: MTC Cueing");
														sprintf(text,"%s [Universal RT: MTC Cueing]",segtype);
														break;
													case 0x08: // MIDI Tunging standard
														sprintf(entry,"Universal RT: MIDI Tuning standard");
														sprintf(text,"%s [Universal RT: MIDI Tuning standard]",segtype);
														break;
													default:
														sprintf(entry,"Universal RT: unknown (0x%02x)",subid1);
														sprintf(text,"%s [Universal RT: unknown (0x%02x)]",segtype,subid1);
												}
												sysexdata_tree = proto_item_add_subtree( command_item, ett_rtp_midi_sysex_data);
												proto_tree_add_uint ( sysexdata_tree, hf_rtp_midi_sysex_chan, tvb, offset+cmd_offset+2, 1, sysexchan);
												command_item = proto_tree_add_text(sysexdata_tree, tvb, offset+cmd_offset+3, 1, entry);
											}
											break;
										default:
											sysexdata_tree = proto_item_add_subtree( command_item, ett_rtp_midi_sysex_data);
											sprintf(text,"Manufacturer: %s",get_short_manu(helptext,manu1));
											command_item = proto_tree_add_text(sysexdata_tree, tvb, offset+cmd_offset+1, 1, text);
											sprintf(text,"Common: System-Exclusive (complete segment) [manu=%s]",get_short_manu(helptext,manu1));
									}

								}
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset+consumed-1, 1, octet2);
								return consumed;
								break;
							case 0xf4:
								consumed++;
								strcpy(text,"Common: System-Exclusive (cancel)");
								command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
								command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
								command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-2, "Sysex-data");
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset+consumed-1, 1, octet2);
								return consumed;
								break;
						}
						break;
					}
					consumed++;
				}
				// if we didn't find proper segment-terminator -> error
				if (TRUE==foundend) {
				  strcpy(text,"Common: System-Exclusive (invalid termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "Sysex-data");
				  return consumed;
				} else {
				  strcpy(text,"Common: System-Exclusive (no termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "Sysex-data");
				  return 0;
				}

				break;
			case 0xf1:
				foundend=FALSE;
				while((cmd_offset+consumed)<cmd_len) {
					octet2 = tvb_get_guint8(tvb, offset+cmd_offset+consumed );
					if ((octet2 & 0x80)==0x80) {
						foundend=TRUE;
						break;
					}
					consumed++;
				}
				// if we didn't find proper segment-terminator -> error
				if (TRUE==foundend) {
				  strcpy(text,"Common: Undefined");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "data");
				  return consumed;
				} else {
				  strcpy(text,"Common: Undefined (no termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "data");
				  return 0;
				}
				break;
			case 0xf2:
				if ((cmd_offset+consumed+2)>cmd_len) {
					strcpy(text,"Common: Song position");
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				octet3=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				wordvalue=octet3 & 0x7f;
				wordvalue<<=7;
				wordvalue|=octet2 & 0x7f;
				sprintf(text,"Common: Song position (0x%04x)",wordvalue);
				command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				sprintf(helptext,"Position pointer: 0x%02x",wordvalue);
				command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, 2, helptext);
				break;
			case 0xf3:
				if ((cmd_offset+consumed+1)>cmd_len) {
					strcpy(text,"Common: Song select");
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				sprintf(text,"Common: Song select (0x%04x)",octet2 &0x7f);
				command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				sprintf(helptext,"Song: 0x%02x",octet2);
				command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, 1, helptext);
				break;
			case 0xf4:
				foundend=FALSE;
				while((cmd_offset+consumed)<cmd_len) {
					octet2 = tvb_get_guint8(tvb, offset+cmd_offset+consumed );
					if ((octet2 & 0x80)==0x80) {
						foundend=TRUE;
						break;
					}
					consumed++;
				}
				// if we didn't find proper segment-terminator -> error
				if (TRUE==foundend) {
				  strcpy(text,"Common: Undefined");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "data");
				  return consumed;
				} else {
				  strcpy(text,"Common: Undefined (no termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "data");
				  return 0;
				}
				break;
			case 0xf5:
				foundend=FALSE;
				while((cmd_offset+consumed)<cmd_len) {
					octet2 = tvb_get_guint8(tvb, offset+cmd_offset+consumed );
					if ((octet2 & 0x80)==0x80) {
						foundend=TRUE;
						break;
					}
					consumed++;
				}
				// if we didn't find proper segment-terminator -> error
				if (TRUE==foundend) {
				  strcpy(text,"Common: Undefined");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "data");
				  return consumed;
				} else {
				  strcpy(text,"Common: Undefined (no termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "data");
				  return 0;
				}
				break;
			case 0xf6:
				  strcpy(text,"Common: Tune request");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				break;
			case 0xf7:
				foundend=FALSE;
				while((cmd_offset+consumed)<cmd_len) {
					octet2 = tvb_get_guint8(tvb, offset+cmd_offset+consumed );
					if ((octet2 & 0x80)==0x80) {
						foundend=TRUE;
						switch (octet2) {
							case 0xf0:
								consumed++;
								strcpy(text,"Common: System-Exclusive (middle segment)");
								command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
								command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
								command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-2, "Sysex-data");
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset+consumed-1, 1, octet2);
								return consumed;
								break;
							case 0xf5:
							case 0xf7:
								consumed++;
								if (octet2==0xf7) {
									strcpy(text,"Common: System-Exclusive (last segment)");
								} else {
									strcpy(text,"Common: System-Exclusive (last segment, non 0xf7-status termination)");
								}
								command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
								command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
								command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-2, "Sysex-data");
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset+consumed-1, 1, octet2);
								return consumed;
								break;
							case 0xf4:
								consumed++;
								strcpy(text,"Common: System-Exclusive (cancel)");
								command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
								command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
								command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-2, "Sysex-data");
								proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset+consumed-1, 1, octet2);
								return consumed;
								break;
						}
						break;
					}
					consumed++;
				}
				// if we didn't find proper segment-terminator -> error
				if (TRUE==foundend) {
				  strcpy(text,"Common: System-Exclusive (invalid termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "Sysex-data");
				  return consumed;
				} else {
				  strcpy(text,"Common: System-Exclusive (no termination)");
				  command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
				  command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
				  proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
				  command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, consumed-1, "Sysex-data");
				  return 0;
				}

				break;
		}
		// common-data resets runningstatus
	} else if (octet>=0x80) {				// channel data
		cmd=octet & 0xf0;
		channel = octet & 0x0f;
		// set running status to current status-octet
		*runningstatus=octet;
		switch(cmd) {
			case 0x80:
				if ((cmd_offset+consumed+2)>cmd_len) {
					sprintf(text,"Voice: Note off (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				octet3=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				sprintf(text,"Voice: Note off (c=%d,n=%d,v=%d)",channel+1,octet2&0x7f,octet3&0x7f);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_note, tvb, offset+cmd_offset, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_velocity, tvb, offset+cmd_offset+1, 1, octet3 & 0x7f);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_note, tvb, offset+cmd_offset+1, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_velocity, tvb, offset+cmd_offset+2, 1, octet3 & 0x7f);
				}

				break;
			case 0x90:
				if ((cmd_offset+consumed+2)>cmd_len) {
					sprintf(text,"Voice: Note on (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				octet3=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				// note on with velocity=0 is interpreted as note off (to keep runningstate)
				if (octet3)
				  sprintf(text,"Voice: Note on (c=%d,n=%d,v=%d)",channel+1,octet2&0x7f,octet3&0x7f);
				else
				  sprintf(text,"Voice: Note off* (c=%d,n=%d,v=%d)",channel+1,octet2&0x7f,octet3&0x7f);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_note, tvb, offset+cmd_offset, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_velocity, tvb, offset+cmd_offset+1, 1, octet3 & 0x7f);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_note, tvb, offset+cmd_offset+1, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_velocity, tvb, offset+cmd_offset+2, 1, octet3 & 0x7f);
				}
				break;
			case 0xa0:
				if ((cmd_offset+consumed+2)>cmd_len) {
					sprintf(text,"Voice: Key pressure (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				octet3=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				sprintf(text,"Voice: Key pressure (c=%d,n=%d,p=%d)",channel+1,octet2&0x7f,octet3&0x7f);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_note, tvb, offset+cmd_offset, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_pressure, tvb, offset+cmd_offset+1, 1, octet3 & 0x7f);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_note, tvb, offset+cmd_offset+1, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_pressure, tvb, offset+cmd_offset+2, 1, octet3 & 0x7f);
				}
				break;
			case 0xb0:
				if ((cmd_offset+consumed+2)>cmd_len) {
					sprintf(text,"Voice: Controller change (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				octet3=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				sprintf(text,"Voice: Controller change (c=%d,\"%s\",v=%d)",channel+1,get_controller(helptext,(guint8)(octet2&0x7f),(guint8)(octet3&0x7f)),octet3&0x7f);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
					sprintf(helptext2,"Controller: %s",get_controller(helptext,(guint8)(octet2&0x7f),(guint8)(octet3&0x7f)));
					proto_tree_add_text ( command_tree, tvb, offset+cmd_offset, 1, helptext2);
//					proto_tree_add_uint ( command_tree, hf_rtp_midi_controller, tvb, offset+cmd_offset, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_controller_value, tvb, offset+cmd_offset+1, 1, octet3 & 0x7f);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
					sprintf(helptext2,"Controller: %s",get_controller(helptext,(guint8)(octet2&0x7f),(guint8)(octet3&0x7f)));
					proto_tree_add_text ( command_tree, tvb, offset+cmd_offset+1, 1, helptext2);
//					proto_tree_add_uint ( command_tree, hf_rtp_midi_controller, tvb, offset+cmd_offset+1, 1, octet2 & 0x7f);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_controller_value, tvb, offset+cmd_offset+2, 1, octet3 & 0x7f);
				}
				break;
			case 0xc0:
				if ((cmd_offset+consumed+1)>cmd_len) {
					sprintf(text,"Voice: Program change (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				sprintf(text,"Voice: Program change (c=%d,p=%d)",channel+1,octet2&0x7f);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_program, tvb, offset+cmd_offset, 1, octet2 & 0x7f);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_program, tvb, offset+cmd_offset+1, 1, octet2 & 0x7f);
				}
				break;
			case 0xd0:
				if ((cmd_offset+consumed+1)>cmd_len) {
					sprintf(text,"Voice: Channel pressure (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				sprintf(text,"Voice: Channel pressure (c=%d,p=%d)",channel+1,octet2&0x7f);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel_pressure, tvb, offset+cmd_offset, 1, octet2 & 0x7f);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel_pressure, tvb, offset+cmd_offset+1, 1, octet2 & 0x7f);
				}
				break;
			case 0xe0:
				if ((cmd_offset+consumed+2)>cmd_len) {
					sprintf(text,"Voice: Pitch bend (c=%d)",channel+1);
					if (usingrs) {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-(cmd_offset+1), text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_rsstatus, tvb, *rsoffset, 1, octet);
					} else {
						command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, cmd_len-cmd_offset, text);
						command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
						proto_tree_add_uint ( command_tree, hf_rtp_midi_status, tvb, offset+cmd_offset, 1, octet);
					}
					return 0;
				}
				octet2=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;
				octet3=tvb_get_guint8(tvb,offset+cmd_offset+consumed);
				consumed++;

				wordvalue=octet3 & 0x7f;
				wordvalue<<=7;
				wordvalue|=octet2 & 0x7f;

				sprintf(text,"Voice: Pitch bend (c=%d,p=%d)",channel+1,wordvalue);
				if (usingrs) {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rscommand, tvb, *rsoffset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_rschannel, tvb, *rsoffset, 1, channel);
				    sprintf(helptext,"Pitchbend: 0x%02x",wordvalue);
				    command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset, 2, helptext);
				} else {
					command_item = proto_tree_add_text(tree, tvb, offset+cmd_offset, consumed, text);
					command_tree = proto_item_add_subtree( command_item, ett_rtp_midi_command);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_command, tvb, offset+cmd_offset, 1, cmd);
					proto_tree_add_uint ( command_tree, hf_rtp_midi_channel, tvb, offset+cmd_offset, 1, channel);
				    sprintf(helptext,"Pitchbend: 0x%02x",wordvalue);
				    command_item = proto_tree_add_text(command_tree, tvb, offset+cmd_offset+1, 2, helptext);
				}
				break;
		}
	} else {								// should never get here...
		return 0;
	}



	return consumed;
}

static void
dissect_rtp_midi( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
{
	proto_item *ti            = NULL;
	proto_item *command_item      = NULL;
	proto_item *journal_item      = NULL;
	proto_tree *rtp_midi_tree     = NULL;
	proto_tree *rtp_midi_commands_tree     = NULL;
	proto_tree *rtp_midi_journal_tree     = NULL;
	proto_tree *rtp_midi_sysjournal_tree     = NULL;
	proto_tree *rtp_midi_chanjournals_tree   = NULL;
	proto_tree *rtp_midi_chanjournal_tree     = NULL;
	unsigned int offset       = 0;
	struct _rtp_conversation_info *p_conv_data = NULL;

	guint8			flags;
	guint8			flags2;
	guint8			octet;
	guint8			totchan;
	guint16			check_seq_num;
	unsigned int    cmd_len;
	unsigned int	cmd_offset;
	unsigned int	cmd_count;
	unsigned int	consumed;
	guint8			runningstatus;
	unsigned int	rsoffset;
	char			text[160];
	char			info[170];
	char			help[100];
	unsigned int	i;
	guint16			sysjourlen;
	guint16			chanjourlen;
	guint16			systemflags;
	guint32			chanflags;
	guint8			channel;



	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )
	  {
	    col_set_str( pinfo->cinfo, COL_PROTOCOL, "RTP-MIDI" );
	  }
	if (check_col(pinfo->cinfo, COL_INFO))
		col_clear(pinfo->cinfo, COL_INFO);


	/* Get event fields */


//	if (tree) {

		// get header flags
		flags = tvb_get_guint8(tvb, offset );

		ti = proto_tree_add_item(tree,proto_rtp_midi,tvb,0,-1,FALSE);
	    rtp_midi_tree = proto_item_add_subtree( ti, ett_rtp_midi);

		proto_tree_add_boolean (rtp_midi_tree, hf_rtp_midi_bflag, tvb, offset, 1, flags);
		proto_tree_add_boolean (rtp_midi_tree, hf_rtp_midi_jflag, tvb, offset, 1, flags);
		proto_tree_add_boolean (rtp_midi_tree, hf_rtp_midi_zflag, tvb, offset, 1, flags);
		proto_tree_add_boolean (rtp_midi_tree, hf_rtp_midi_pflag, tvb, offset, 1, flags);

		cmd_len=flags & 0xf;

		// see if we have small or large len-field
		if ((flags & 0x80)==0x80)  {
			octet = tvb_get_guint8(tvb, offset+1 );
			cmd_len=(cmd_len << 8) | octet;
		    proto_tree_add_uint ( rtp_midi_tree, hf_rtp_midi_longlen, tvb, offset, 2, cmd_len);
			offset+=2;
		} else {
			proto_tree_add_uint ( rtp_midi_tree, hf_rtp_midi_shortlen, tvb, offset, 1, cmd_len);
			offset++;
		}

		// if we have a command-section -> dissect it
		if (cmd_len) {
	
		    command_item = proto_tree_add_text(rtp_midi_tree, tvb, offset, cmd_len, "Commands");
	        rtp_midi_commands_tree = proto_item_add_subtree( command_item, ett_rtp_midi_commands);

			// if the reported command-length larger than data found in packet -> error
			if ((offset+cmd_len)>tvb->length) {
				THROW(ReportedBoundsError);
				return;
			}

			cmd_offset=0;
		    cmd_count=0;
			// RTP-pdus always start with no running status
			runningstatus=0;
		    while (cmd_offset<cmd_len) {

				
				// for the first command we only have a delta-time if Z-Flag is set
				if ((cmd_count!=0) || ((flags&0x20)==0x20)) {
					
					consumed=decodetime(tvb, pinfo, rtp_midi_commands_tree, offset, cmd_offset, cmd_len);
					if (0==consumed) {
						THROW(ReportedBoundsError);
						return;
					}
					cmd_offset+=consumed;

					// we should have at least one midi-command coming after delta-time
					if (cmd_offset>cmd_len) {
						THROW(ReportedBoundsError);
						return;
					}
				}

				// dissect midi-command
				consumed=decodemidi(tvb, pinfo, rtp_midi_commands_tree, offset, cmd_offset, cmd_len, &runningstatus, &rsoffset, text);
				if (0==consumed) {
					THROW(ReportedBoundsError);
					return;
				}
				cmd_offset+=consumed;
				
				cmd_count++;

				if ( check_col( pinfo->cinfo, COL_INFO) ) {
					if (1==cmd_count) {
						strcpy(info,text);
						col_add_fstr(pinfo->cinfo, COL_INFO,text);
					} else {
						if (2==cmd_count) {
							sprintf(text,"%s - one more command",info);
						} else {
							sprintf(text,"%s - %d more commands",info,cmd_count-1);
						}
						col_add_fstr(pinfo->cinfo, COL_INFO,text);
					}
				}
			}
			offset+=cmd_len;

		}

		// if we have a journal-section -> dissect it
		if ((flags & 0x40)==0x40) {
		  journal_item = proto_tree_add_text(rtp_midi_tree, tvb, offset, -1, "Journal");
	      rtp_midi_journal_tree = proto_item_add_subtree( journal_item, ett_rtp_midi_journal);
		  flags2 = tvb_get_guint8(tvb, offset );
		  proto_tree_add_boolean (rtp_midi_journal_tree, hf_rtp_midi_sflag, tvb, offset, 1, flags2);
		  proto_tree_add_boolean (rtp_midi_journal_tree, hf_rtp_midi_yflag, tvb, offset, 1, flags2);
		  proto_tree_add_boolean (rtp_midi_journal_tree, hf_rtp_midi_aflag, tvb, offset, 1, flags2);
		  proto_tree_add_boolean (rtp_midi_journal_tree, hf_rtp_midi_hflag, tvb, offset, 1, flags2);
		  totchan=flags2&0xf;
		  proto_tree_add_uint ( rtp_midi_journal_tree, hf_rtp_midi_totchan, tvb, offset, 1, totchan);
		  offset++;
		  check_seq_num=tvb_get_ntohs(tvb,offset);
		  proto_tree_add_uint ( rtp_midi_journal_tree, hf_rtp_midi_check_seq_num, tvb, offset, 2, check_seq_num);
		  offset+=2;

		  // do we have system journal?
		  if ((flags2 & 0x40)==0x40) {
				systemflags=tvb_get_ntohs(tvb,offset);
				sysjourlen=systemflags&0x3ff;
				ti = proto_tree_add_text(rtp_midi_journal_tree, tvb, offset, sysjourlen+2, "System-Journal");
				rtp_midi_sysjournal_tree = proto_item_add_subtree( ti, ett_rtp_midi_systemjournal);
				proto_tree_add_boolean (rtp_midi_sysjournal_tree, hf_rtp_midi_sysjour_toc_s, tvb, offset, 2, systemflags);
				proto_tree_add_boolean (rtp_midi_sysjournal_tree, hf_rtp_midi_sysjour_toc_d, tvb, offset, 2, systemflags);
				proto_tree_add_boolean (rtp_midi_sysjournal_tree, hf_rtp_midi_sysjour_toc_v, tvb, offset, 2, systemflags);
				proto_tree_add_boolean (rtp_midi_sysjournal_tree, hf_rtp_midi_sysjour_toc_q, tvb, offset, 2, systemflags);
				proto_tree_add_boolean (rtp_midi_sysjournal_tree, hf_rtp_midi_sysjour_toc_f, tvb, offset, 2, systemflags);
				proto_tree_add_boolean (rtp_midi_sysjournal_tree, hf_rtp_midi_sysjour_toc_x, tvb, offset, 2, systemflags);
				proto_tree_add_uint ( rtp_midi_journal_tree, hf_rtp_midi_totchan, tvb, offset, 2, systemflags);
				offset+=2;
				ti = proto_tree_add_text(rtp_midi_sysjournal_tree, tvb, offset, sysjourlen-2, "System Chapters");
		  }

		  // do we have channel journal?
		  if ((flags2 & 0x20)==0x20) {
			  	ti = proto_tree_add_text(rtp_midi_journal_tree, tvb, offset, -1, "Channel-Journal");
				rtp_midi_chanjournals_tree = proto_item_add_subtree( ti, ett_rtp_midi_channeljournals);
				// iterate through all the channels specified in header
				for (i=0;i<=totchan;i++) {
					chanflags=tvb_get_ntoh24(tvb,offset);
					channel=(chanflags>>19)&0xf;
					chanjourlen=(chanflags>>8)&0x3ff;
					sprintf(help,"Channel %d",channel+1);
			  		ti = proto_tree_add_text(rtp_midi_chanjournals_tree, tvb, offset, chanjourlen, help);
					rtp_midi_chanjournal_tree = proto_item_add_subtree( ti, ett_rtp_midi_channeljournal);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_sflag, tvb, offset, 3, chanflags);
				    proto_tree_add_uint ( rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_chan, tvb, offset, 3,  chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_hflag, tvb, offset, 3, chanflags);
				    proto_tree_add_uint ( rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_len, tvb, offset, 3,  chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_p, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_c, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_m, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_w, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_n, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_e, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_t, tvb, offset, 3, chanflags);
					proto_tree_add_boolean (rtp_midi_chanjournal_tree, hf_rtp_midi_chanjour_toc_a, tvb, offset, 3, chanflags);
					ti = proto_tree_add_text(rtp_midi_chanjournal_tree, tvb, offset+3, chanjourlen-3, "Channel Chapters");
					offset+=chanjourlen;
				}
		  }

		}
//	} // if (tree)

}




void
proto_register_rtp_midi(void)
{

	module_t *rtp_midi_module;

	static hf_register_info hf[] =
	{
		{
			&hf_rtp_midi_bflag,
			{
				"B-Flag",
				"rtpmidi.b_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x80,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_jflag,
			{
				"J-Flag",
				"rtpmidi.j_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x40,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_zflag,
			{
				"Z-Flag",
				"rtpmidi.z_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x20,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_pflag,
			{
				"P-Flag",
				"rtpmidi.p_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x10,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_shortlen,
			{
				"Command length (short)",
				"rtpmidi.cmd_length_short",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x0F,
				"", HFILL
			}
		},

		{
			&hf_rtp_midi_longlen,
			{
				"Command length (long)",
				"rtpmidi.cmd_length_long",
				FT_UINT16,
				BASE_DEC,
				NULL,
				0x0FFF,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sflag,
			{
				"S-Flag",
				"rtpmidi.s_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x80,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_yflag,
			{
				"Y-Flag",
				"rtpmidi.y_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x40,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_aflag,
			{
				"A-Flag",
				"rtpmidi.a_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x20,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_hflag,
			{
				"H-Flag",
				"rtpmidi.h_flag",
				FT_BOOLEAN,
				8,
				NULL,
				0x10,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_totchan,
			{
				"Total channels",
				"rtpmidi.total_channels",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x0F,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_check_seq_num,
			{
				"Checkpoint Packet Seqnum",
				"rtpmidi.check_Seq_num",
				FT_UINT16,
				BASE_DEC,
				NULL,
				0x00,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_deltatime1,
			{
				"Delta Time (one octet)",
				"rtpmidi.deltatime_1",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_deltatime2,
			{
				"Delta Time (two octets)",
				"rtpmidi.deltatime_2",
				FT_UINT16,
				BASE_HEX,
				NULL,
				0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_deltatime3,
			{
				"Delta Time (three octets)",
				"rtpmidi.deltatime_3",
				FT_UINT24,
				BASE_HEX,
				NULL,
				0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_deltatime4,
			{
				"Delta Time (four octets)",
				"rtpmidi.deltatime_4",
				FT_UINT32,
				BASE_HEX,
				NULL,
				0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_status,
			{
				"Status",
				"rtpmidi.status",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_rsstatus,
			{
				"RS Status",
				"rtpmidi.rsstatus",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_command,
			{
				"Command",
				"rtpmidi.command",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0xf0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_rscommand,
			{
				"RS Command",
				"rtpmidi.rscommand",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0xf0,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_channel,
			{
				"Channel",
				"rtpmidi.channel",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0x0f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_rschannel,
			{
				"RS Channel",
				"rtpmidi.rschannel",
				FT_UINT8,
				BASE_HEX,
				NULL,
				0x0f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_note,
			{
				"Note",
				"rtpmidi.note",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_velocity,
			{
				"Velocity",
				"rtpmidi.velocity",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_pressure,
			{
				"Pressure",
				"rtpmidi.pressure",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},
/*		{
			&hf_rtp_midi_controller,
			{
				"Controller",
				"rtpmidi.controller",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},*/
		{
			&hf_rtp_midi_controller_value,
			{
				"Value",
				"rtpmidi.controller_value",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_program,
			{
				"Program",
				"rtpmidi.program",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_channel_pressure,
			{
				"Pressure",
				"rtpmidi.channel_pressure",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0x7f,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_toc_s,
			{
				"System-Journal TOC-S-Flag",
				"rtpmidi.sysjour_toc_s",
				FT_BOOLEAN,
				16,
				NULL,
				0x8000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_toc_d,
			{
				"System-Journal TOC-D-Flag",
				"rtpmidi.sysjour_toc_d",
				FT_BOOLEAN,
				16,
				NULL,
				0x4000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_toc_v,
			{
				"System-Journal TOC-V-Flag",
				"rtpmidi.sysjour_toc_v",
				FT_BOOLEAN,
				16,
				NULL,
				0x2000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_toc_q,
			{
				"System-Journal TOC-Q-Flag",
				"rtpmidi.sysjour_toc_q",
				FT_BOOLEAN,
				16,
				NULL,
				0x1000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_toc_f,
			{
				"System-Journal TOC-F-Flag",
				"rtpmidi.sysjour_toc_f",
				FT_BOOLEAN,
				16,
				NULL,
				0x0800,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_toc_x,
			{
				"System-Journal TOC-X-Flag",
				"rtpmidi.sysjour_toc_x",
				FT_BOOLEAN,
				16,
				NULL,
				0x0400,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysjour_len,
			{
				"System-Journal-Length",
				"rtpmidi.cmd_sysjour_len",
				FT_UINT16,
				BASE_DEC,
				NULL,
				0x03FF,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_sflag,
			{
				"Channel-Journal S-Flag",
				"rtpmidi.chanjour_s",
				FT_BOOLEAN,
				24,
				NULL,
				0x800000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_chan,
			{
				"Channel",
				"rtpmidi.chanjour_channel",
				FT_UINT24,
				BASE_HEX,
				NULL,
				0x780000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_hflag,
			{
				"Channel-Journal H-Flag",
				"rtpmidi.chanjour_h",
				FT_BOOLEAN,
				24,
				NULL,
				0x040000,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_len,
			{
				"Channel-Journal-Length",
				"rtpmidi.cmd_chanjour_len",
				FT_UINT24,
				BASE_DEC,
				NULL,
				0x03FF00,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_p,
			{
				"Channel-Journal TOC-P-Flag",
				"rtpmidi.chanjour_toc_p",
				FT_BOOLEAN,
				24,
				NULL,
				0x80,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_c,
			{
				"Channel-Journal TOC-C-Flag",
				"rtpmidi.chanjour_toc_c",
				FT_BOOLEAN,
				24,
				NULL,
				0x40,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_m,
			{
				"Channel-Journal TOC-M-Flag",
				"rtpmidi.chanjour_toc_m",
				FT_BOOLEAN,
				24,
				NULL,
				0x20,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_w,
			{
				"Channel-Journal TOC-W-Flag",
				"rtpmidi.chanjour_toc_w",
				FT_BOOLEAN,
				24,
				NULL,
				0x10,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_n,
			{
				"Channel-Journal TOC-N-Flag",
				"rtpmidi.chanjour_toc_n",
				FT_BOOLEAN,
				24,
				NULL,
				0x08,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_e,
			{
				"Channel-Journal TOC-E-Flag",
				"rtpmidi.chanjour_toc_e",
				FT_BOOLEAN,
				24,
				NULL,
				0x04,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_t,
			{
				"Channel-Journal TOC-T-Flag",
				"rtpmidi.chanjour_toc_t",
				FT_BOOLEAN,
				24,
				NULL,
				0x02,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_chanjour_toc_a,
			{
				"Channel-Journal TOC-A-Flag",
				"rtpmidi.chanjour_toc_a",
				FT_BOOLEAN,
				24,
				NULL,
				0x01,
				"", HFILL
			}
		},
		{
			&hf_rtp_midi_sysex_chan,
			{
				"Sysex-Channel",
				"rtpmidi.sysex_chan",
				FT_UINT8,
				BASE_DEC,
				NULL,
				0,
				"", HFILL
			}
		},

	};


	static gint *ett[] =
	{
		&ett_rtp_midi,
		&ett_rtp_midi_commands,
		&ett_rtp_midi_journal,
		&ett_rtp_midi_command,
		&ett_rtp_midi_systemjournal,
		&ett_rtp_midi_channeljournals,
		&ett_rtp_midi_systemchapters,
		&ett_rtp_midi_channeljournal,
		&ett_rtp_midi_channelchapters,
		&ett_rtp_midi_sysex_data,
	};





	proto_rtp_midi = proto_register_protocol("RTP-MIDI", "RTP-MIDI", "rtpmidi");
	proto_register_field_array(proto_rtp_midi, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));


    /* Register preferences */
    rtp_midi_module = prefs_register_protocol (proto_rtp_midi, proto_reg_handoff_rtp_midi);
    prefs_register_uint_preference (rtp_midi_module,
                                    "midi_payload_type_value", "Payload Type for RTP-MIDI",
                                    "This is the value of the Payload Type field "
                                    "that specifies RTP-MIDI", 10,
                                    &rtp_midi_payload_type_value);
	register_dissector("rtpmidi", dissect_rtp_midi, proto_rtp_midi);
}



void
proto_reg_handoff_rtp_midi(void)
{
	static dissector_handle_t rtp_midi_handle;
	static int rtp_midi_prefs_initialized = FALSE;


  	if (!rtp_midi_prefs_initialized) {
		rtp_midi_handle = create_dissector_handle(dissect_rtp_midi, proto_rtp_midi);
		rtp_midi_prefs_initialized = TRUE;
	}
	else {
		dissector_delete("rtp.pt", saved_payload_type_value, rtp_midi_handle);
	}

	saved_payload_type_value = rtp_midi_payload_type_value;
	/* rtp_event_payload_type_value is set from preferences */

    dissector_add("rtp.pt", saved_payload_type_value, rtp_midi_handle);
	dissector_add_string("rtp_dyn_payload_type", "midi", rtp_midi_handle);
}
